import { NavigationService } from './../../services/navigation.service';
import { TokenService } from 'src/app/services/token.service';
import { OrderService } from './../../../openapi/generated/services/order.service';
//import { TokenCookieService } from 'src/app/services/token-cookie.service';
import { ManagementService } from 'src/openapi/generated/services/management.service';
import { Component, OnDestroy, OnInit, ViewChild } from '@angular/core';
import { TranslateService } from '@ngx-translate/core';
import { FaudokResponsePageOrderResponseDto, FaudokUserInfoResponseDto, OrderResponseDto } from 'src/openapi/generated/models';
import { Router } from '@angular/router';
import { AlertService } from 'src/app/services/alert.service';
import { LoadingService } from 'src/app/services/loading.service';
import { Subscription } from 'rxjs/internal/Subscription';
import { AlertType } from 'src/app/alert/alert-type.enum';
import { MatSidenav } from '@angular/material/sidenav';
import { BreakpointObserver } from '@angular/cdk/layout';
import { filter, map, Observable, switchMap, tap } from 'rxjs';
import { MatDialog } from '@angular/material/dialog';
import { ConfirmationDialogComponent } from 'src/app/confirmation-dialog/confirmation-dialog.component';

@Component({
  selector: 'app-header',
  templateUrl: './header.component.html',
  styleUrls: ['./header.component.css']
})
export class HeaderComponent implements OnInit, OnDestroy {
  
  username: string = '';
  isUserLoggedIn: boolean = false;
  isAdmin$!: Observable<boolean>;
  searchedOrders: OrderResponseDto[] = [];
  currentFaculty = '';
  faculties: string[] = [];
  selectedLanguage = 'de';
  opened = false;
  isDesktop: boolean = true;

  @ViewChild(MatSidenav)
  sidenav!: MatSidenav;  //to monitor the sidenav
  isMobile= true;
  isCollapsed = true;
  private subscriptions: Subscription[] = [];


  constructor(
    private navigationService: NavigationService,
    private translateService: TranslateService, 
    private managementService: ManagementService,
    private router: Router,
    private alertService: AlertService,
    private loadingService: LoadingService,
    private tokenService: TokenService,
    private observer: BreakpointObserver,
  ) {
    this.translateService.setDefaultLang(this.selectedLanguage);
  }


  ngOnInit() {
    this.isAdmin$ = this.tokenService.userAuthority$.pipe(
      map(authority => authority == 'ROLE_ADMIN')
    );
    //check screen size to change layout of navbar menu
    this.subscriptions.push(
      this.observer.observe(['(max-width: 800px)']).subscribe((screenSize) => {
        if(screenSize.matches){
          this.isMobile = true;
        } else {
          this.isMobile = false;
        }
      })
    );
    //if log in/out, update the information in navbar
    this.subscriptions.push(
      this.tokenService.isLoggedInSubject
      .subscribe(
        isLoggedIn => {
          this.isUserLoggedIn = isLoggedIn;
          // Clear user data when logged out
          if (!isLoggedIn) {
            this.username = '';
            this.faculties = [];
            this.currentFaculty = '';
          } else {
            // Only fetch account details when logged in
            this.loadAccountDetails();
          }
        }
      )
    );

    // Initialize the login state check
    this.tokenService.isTokenValid();
  }

  private loadAccountDetails() {
    this.subscriptions.push(
      this.managementService.getAccountDetails().subscribe({
        next: (accountDetails: FaudokUserInfoResponseDto) => {
          this.username = accountDetails.name;
          this.faculties = accountDetails.availableFaculties;
          this.currentFaculty = accountDetails.currentFaculty;
        },
        error: () => {
          // Handle API error - possibly token is invalid
          this.tokenService.removeToken();
          this.router.navigate(['/login']);
          this.alertService.showAlert('Error while loading user details.', AlertType.ERROR);
        }
      })
    );
  }


  onFacultySelected(faculty: string) {
    this.loadingService.isLoading.next(true);
    this.subscriptions.push(
      this.managementService.updateCurrentFaculty({ body: { mainFaculty: faculty}}).subscribe(
        () => {
          this.loadingService.isLoading.next(false);
          this.currentFaculty = faculty;
          this.alertService.showAlert('Your current faculty has been updated.', AlertType.SUCCESS);
          this.router.navigate(['/']);
        },
        (error) => {
          this.loadingService.isLoading.next(false);
          this.alertService.showAlert('Updating your current faculty failed. Please try again.', AlertType.ERROR);
        }
      )

    );
  }

  toggleMenu() {
    if(this.isMobile){
      this.sidenav.toggle();
      this.isCollapsed = false; // On mobile, the menu can never be collapsed
    } else {
      this.sidenav.open(); // On desktop/tablet, the menu can never be fully closed
      this.isCollapsed = !this.isCollapsed;
    }
  }

  switchLanguage(language) {
    this.selectedLanguage = language;
    this.translateService.use(this.selectedLanguage);
  }
  
  logout(): void {
    this.navigationService.setLogoutPending(true); // Set flag to skip modal
    this.router.navigate(['/login'], { replaceUrl: true }).then(() => {
      this.tokenService.removeToken();
      this.alertService.showAlert('You have been successfully logged out.', AlertType.SUCCESS);
      this.navigationService.setLogoutPending(false); // Reset flag after navigation
    });
  }

  ngOnDestroy() {
    this.subscriptions.forEach(sub => sub.unsubscribe);
  }

}
