import { environment } from 'src/environments/environment';
import { Component, OnDestroy, OnInit } from '@angular/core';
import { Router } from '@angular/router';
import { AlertService } from 'src/app/services/alert.service';
import { LoadingService } from 'src/app/services/loading.service';
import { TokenService } from 'src/app/services/token.service';
import { LoginRequestDto } from 'src/openapi/generated/models';
import { AuthenticationService } from 'src/openapi/generated/services/authentication.service';
import { Subscription } from 'rxjs/internal/Subscription';
import { AlertType } from 'src/app/alert/alert-type.enum';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';


@Component({
  selector: 'app-login',
  templateUrl: './login.component.html',
  styleUrls: ['./login.component.css']
})
export class LoginComponent implements OnInit, OnDestroy {

loginForm: FormGroup;
private subscriptions: Subscription[] = [];


constructor(
  private loadingService: LoadingService,
  private alertService: AlertService,
  private router: Router,
  private fb: FormBuilder,
  private authService: AuthenticationService,
  //TODO use TokenCookieService as is more secure
  // private tokencookieService: TokenCookieService,
  private tokenService: TokenService,
) {
}

ngOnInit(): void {
  this.loginForm = this.fb.group({
    email: ['', [Validators.required, Validators.email]],
    password: ['', [Validators.required, Validators.minLength(8), Validators.maxLength(62)]],
  });
  if (this.tokenService.isLoggedInSubject.getValue()) {
    //if already loggedin, go directly to requested page
    if (this.tokenService.redirectUrl) {
      this.router.navigateByUrl(this.tokenService.redirectUrl);
    } else {
      //or if no specific page, go directly to orders
      this.router.navigateByUrl('/orders');
    }
  } else {
    this.router.navigateByUrl('/login');
  }
}


onSubmit() {
  if (this.loginForm.valid) {
    const email = this.loginForm.get('email').value;
    const password = this.loginForm.get('password').value;
    this.loadingService.isLoading.next(true);
    this.subscriptions.push(
      this.authService.login({ body: { email: email, password: password } }).subscribe({
        next: (res) => {
          this.loadingService.isLoading.next(false);
          this.alertService.showAlert('Login success.', AlertType.SUCCESS );
          this.tokenService.token = res.token as string;
          this.tokenService.isTokenValid();
          this.router.navigate(['orders']);
        },
        error: (err) => {
          this.loadingService.isLoading.next(false);
          this.alertService.showAlert('Login failed. Please try again.', AlertType.ERROR );
          this.tokenService.removeToken();
        }
      })
    );
  }
}

onForgotPassword(): void {
  this.router.navigate(['forgot-password']);
}

ngOnDestroy() {
  this.subscriptions.forEach(sub => sub.unsubscribe());
}

}
