import { AfterViewInit, Component, OnDestroy, ViewChild } from '@angular/core';
import { MatDialog } from '@angular/material/dialog';
import { MatPaginator } from '@angular/material/paginator';
import { MatSort } from '@angular/material/sort';
import { MatTableDataSource } from '@angular/material/table';
import { Router } from '@angular/router';
import { Subscription, merge, startWith, switchMap, catchError, of, map, Observable } from 'rxjs';
import { AlertType } from 'src/app/alert/alert-type.enum';
import { AlertService } from 'src/app/services/alert.service';
import { LoadingService } from 'src/app/services/loading.service';
import { GetAllArchivedOrders$Params } from 'src/openapi/generated/fn/admin-order/get-all-archived-orders';
import { OrderResponseDto } from 'src/openapi/generated/models';
import { AdminOrderService } from 'src/openapi/generated/services/admin-order.service';

@Component({
  selector: 'app-all-archived-orders',
  templateUrl: './all-archived-orders.component.html',
  styleUrls: ['./all-archived-orders.component.css']
})
export class AllArchivedOrdersComponent implements AfterViewInit, OnDestroy {

  displayedColumns: string[] = ['status', 'id', 'doneDate', 'clientDeliveryEmail', 'signature', 'scanComment', 'author', 'title', 'bvNr', 'isbnissn', 'actions'];
  dataSource: MatTableDataSource<OrderResponseDto> = new MatTableDataSource<OrderResponseDto>();
  resultsLength = 0;
  @ViewChild(MatPaginator) paginator: MatPaginator;
  @ViewChild(MatSort) sort: MatSort;
  subscriptions: Subscription[] = [];


  constructor(
    private loadingService: LoadingService,
    private alertService: AlertService,
    private adminOrderService: AdminOrderService,
    private router: Router,
    public dialog: MatDialog,
  ) {
  }

  ngAfterViewInit() {
    //if table is not created bec has no orders, skip checking pagination and sorting
    if (this.sort && this.paginator) {
      // If the user changes the sort order, reset back to the first page.
      this.sort.sortChange.subscribe(() => (this.paginator.pageIndex = 0));
      this.subscriptions.push(
        //check if sort or pagination observable values changes to update the table
        merge(this.sort.sortChange, this.paginator.page)
          .pipe(
            startWith({}),
            switchMap(() => {
              this.loadingService.isLoading.next(true);
              return this.findAllArchivedOrders().pipe(catchError(() => { this.alertService.showAlert('Could not load orders', AlertType.ERROR); return of(null); }));
            }),
            map(data => {
              // Flip flag to show that loading has finished.
              this.loadingService.isLoading.next(false);
              if (data === null) {
                return [];
              }
              // Only refresh the result length if there is new data. 
              this.resultsLength = data.totalElements;
              return data.content;
            }),
          )
          .subscribe(data => (this.dataSource.data = data ))
      );
    }
  }

  applyFilter(event: Event) {
    const filterValue = (event.target as HTMLInputElement).value;
    this.dataSource.filter = filterValue.trim().toLowerCase();
    if (this.dataSource.paginator) {
      this.dataSource.paginator.firstPage();
    }
  }

  private findAllArchivedOrders() : Observable<any> {
    let params: GetAllArchivedOrders$Params = {
      pageNo: this.paginator.pageIndex,
      pageSize: this.paginator.pageSize,
      sortBy: this.sort.active === 'signature' ? 'document.signature' : this.sort.active,
      sortDir: this.sort.direction,
    };
    return this.adminOrderService.getAllArchivedOrders(params)
  }

  viewOrder(order: OrderResponseDto) {
    this.router.navigate(['orders','archived' ,'view', order.id]);
  }
  
  ngOnDestroy(): void {
    this.subscriptions.forEach((sub) => sub.unsubscribe());
  }
  
}