import { AfterViewInit, Component, EventEmitter, OnDestroy, OnInit, ViewChild } from '@angular/core';
import { MatDialog } from '@angular/material/dialog';
import { MatPaginator, PageEvent } from '@angular/material/paginator';
import { MatSort, Sort } from '@angular/material/sort';
import { MatTableDataSource } from '@angular/material/table';
import { Router } from '@angular/router';
import { catchError, map, merge, Observable, of, startWith, Subscription, switchMap } from 'rxjs';
import { AlertType } from 'src/app/alert/alert-type.enum';
import { AlertService } from 'src/app/services/alert.service';
import { LoadingService } from 'src/app/services/loading.service';
import { TokenService } from 'src/app/services/token.service';
import { FaudokResponsePageOrderResponseDto, OrderResponseDto } from 'src/openapi/generated/models';
import { OrderService, AdminOrderService } from 'src/openapi/generated/services';

@Component({
  selector: 'app-all-available-orders',
  templateUrl: './all-available-orders.component.html',
  styleUrls: ['./all-available-orders.component.css']
})
export class AllAvailableOrdersComponent implements OnDestroy, AfterViewInit {

  displayedColumns: string[] = ['id', 'dueDate', 'clientDeliveryEmail', 'signature', 'author', 'title', 'bvNr', 'isbnissn', 'actions'];
  dataSource: MatTableDataSource<OrderResponseDto> = new MatTableDataSource<OrderResponseDto>();
  resultsLength = 0;
  @ViewChild(MatPaginator) paginator: MatPaginator;
  @ViewChild(MatSort) sort: MatSort;
  subscriptions: Subscription[] = [];


  constructor(
    private loadingService: LoadingService,
    private alertService: AlertService,
    private adminOrderService: AdminOrderService,
    private router: Router,
    private dialog: MatDialog,
    private tokenService: TokenService
  ) {
  }

  ngAfterViewInit() {
    console.log('set false in allAvailOrders');
    this.tokenService.setEditView(false);
    //if table is not created bec has no orders, skip checking pagination and sorting
    if (this.sort && this.paginator) {
      // If the user changes the sort order, reset back to the first page.
      this.sort.sortChange.subscribe(() => (this.paginator.pageIndex = 0));
      this.subscriptions.push(
        //check if sort or pagination observable values changes to update the table
        merge(this.sort.sortChange, this.paginator.page)
          .pipe(
            startWith({}),
            switchMap(() => {
              this.loadingService.isLoading.next(true);
              return this.findAllAvailableOrders().pipe(catchError(() => { this.alertService.showAlert('Could not load orders', AlertType.ERROR); return of(null); }));
            }),
            map(data => {
              // Flip flag to show that loading has finished.
              this.loadingService.isLoading.next(false);
              if (data === null) {
                return [];
              }
              // Only refresh the result length if there is new data. 
              this.resultsLength = data.totalElements;
              return data.content;
            }),
          )
          .subscribe(data => (this.dataSource.data = data ))
      );
    }
  }

  private resetPaginationAndSorting(): void {
    this.paginator.pageIndex = 0;
    this.paginator.pageSize = 5;
    this.sort.active = 'dueDate';
    this.sort.direction = 'asc';
  }

  applyFilter(event: Event) {
    const filterValue = (event.target as HTMLInputElement).value;
    this.dataSource.filter = filterValue.trim().toLowerCase();
    if (this.dataSource.paginator) {
      this.dataSource.paginator.firstPage();
    }
  }

  private findAllAvailableOrders() : Observable<any> {
    let params: any = {
      pageNo: this.paginator.pageIndex,
      pageSize: this.paginator.pageSize,
      sortBy: this.sort.active === 'signature' ? 'document.signature' : this.sort.active,
      sortDir: this.sort.direction,
    };
    return this.adminOrderService.getAllAvailableOrders(params)
  }

  viewOrder(order: OrderResponseDto) {
    this.router.navigate(['orders', 'manage', order.id]); 
  }

  isToday(date: Date | string): boolean {
    return new Date().toDateString() === new Date(date).toDateString();
  }

  ngOnDestroy(): void {
    // this.tokenService.setEditView(true);
    this.subscriptions.forEach((sub) => sub.unsubscribe());
  }

}

