import { AlertService } from 'src/app/services/alert.service';
import { AfterViewInit, Component, OnDestroy, ViewChild } from '@angular/core';
import { Router } from '@angular/router';
import { GetChairsArchivedOrders$Params } from 'src/openapi/generated/fn/order/get-chairs-archived-orders';
import { OrderResponseDto } from 'src/openapi/generated/models';
import { OrderService } from 'src/openapi/generated/services/order.service';
import { LoadingService } from 'src/app/services/loading.service';
import { AlertType } from 'src/app/alert/alert-type.enum';
import { catchError, map, merge, Observable, of, startWith, Subscription, switchMap } from 'rxjs';
import { MatDialog } from '@angular/material/dialog';
import { MatPaginator } from '@angular/material/paginator';
import { MatSort } from '@angular/material/sort';
import { MatTableDataSource } from '@angular/material/table';

@Component({
  selector: 'app-chairs-archived-orders',
  templateUrl: './chairs-archived-orders.component.html',
  styleUrls: ['./chairs-archived-orders.component.css']
})
export class ChairsArchivedOrdersComponent implements OnDestroy, AfterViewInit {


  displayedColumns: string[] = ['status', 'id', 'doneDate', 'clientDeliveryEmail', 'signature', 'scanComment', 'author',  'title', 'bvNr', 'isbnissn', 'actions'];
  //displayedColumns: string[] = ['status', 'id', 'doneDate', 'clientDeliveryEmail', 'signature', 'scanComment', 'authorArticle', 'authorBook', 'titleArticle', 'titleBook', 'bvNr', 'isbn', 'issn', 'actions'];
  dataSource: MatTableDataSource<OrderResponseDto> = new MatTableDataSource<OrderResponseDto>();
  resultsLength = 0;
  @ViewChild(MatPaginator) paginator: MatPaginator;
  @ViewChild(MatSort) sort: MatSort;
  subscriptions: Subscription[] = [];


  constructor(
    private loadingService: LoadingService,
    private alertService: AlertService,
    private orderService: OrderService,
    private router: Router,
    public dialog: MatDialog,
  ) {
    // this.subscriptions.push(
    //   this.orderService.getChairsArchivedOrders({pageNo: 0, pageSize: 5, sortBy: 'doneDate', sortDir: 'desc'}).subscribe(data =>
    //    {
    //     data.totalElements !== 0 ? this.hasOrders = true : this.hasOrders = false;
    //     this.data = new MatTableDataSource(data.content);
    //    }
    //   )
    // );
  }
  
  ngAfterViewInit() {
    //if table is not created because it has no orders, skip checking pagination and sorting
    if (this.sort && this.paginator) {
      // If the user changes the sort order, reset back to the first page.
      this.sort.sortChange.subscribe(() => (this.paginator.pageIndex = 0));
      this.subscriptions.push(
        //check if sort or pagination observable values changes to update the table
        merge(this.sort.sortChange, this.paginator.page)
          .pipe(
            startWith({}),
            switchMap(() => {
              this.loadingService.isLoading.next(true);
              return this.findChairsArchivedOrders().pipe(catchError(() => { this.alertService.showAlert('Could not load orders', AlertType.ERROR); return of(null); }));
            }),
            map(data => {
              // Flip flag to show that loading has finished.
              this.loadingService.isLoading.next(false);
              if (data === null) {
                return [];
              }
              // Only refresh the result length if there is new data. 
              this.resultsLength = data.totalElements;
              return data.content;
            }),
          )
          .subscribe(data => (this.dataSource.data = data ))
      );
    }
  }

  applyFilter(event: Event) {
    const filterValue = (event.target as HTMLInputElement).value;
    this.dataSource.filter = filterValue.trim().toLowerCase();
    if (this.dataSource.paginator) {
      this.dataSource.paginator.firstPage();
    }
  }

  private findChairsArchivedOrders() : Observable<any> {
    let params: GetChairsArchivedOrders$Params = {
      pageNo: this.paginator.pageIndex,
      pageSize: this.paginator.pageSize,
      sortBy: this.sort.active === 'signature' ? 'document.signature' : this.sort.active,
      sortDir: this.sort.direction,
    };
    return this.orderService.getChairsArchivedOrders(params)
  }

  viewOrder(order: OrderResponseDto) {
    this.router.navigate(['orders','archived' ,'view', order.id]);
  }

  scanOrder(order: OrderResponseDto) {
    this.router.navigate(['orders', 'scan', order.id]);
  }

  /**
 * Handles rescanning or merging operations for the specified order.
 * Navigates to the appropriate page based on the operation type.
 * 
 * @param order - The order to be processed, represented by an OrderResponseDto object.
 * @param operation - The operation type ('newScan' or 'mergeToScan').
 */
  handleOrderRescan(order: OrderResponseDto, operation: 'newScan' | 'mergeToScan') {
    if (order.scanned) {
      this.loadingService.isLoading.next(true);
      this.subscriptions.push(
        this.orderService.handleRescanFile({ orderId: order.id }).subscribe({
          next: (res) => {
            // Determine the redirect route based on the operation type
            const route = operation === 'newScan' ? ['orders', 'scan', order.id] : ['orders', 'scan', 'merge', order.id]; 
            //update state to INITIALIZED, remove doneDate etc, rename file to .pdfOLD, to allow merged file to replace old file
            this.router.navigate(route);
            this.loadingService.isLoading.next(false);
          },
          error: (err) => {
            this.loadingService.isLoading.next(false);
            this.alertService.showAlert('Rescan Failed. No scan found for this order!', AlertType.ERROR);
          },
        })
      );
    } else {
      this.alertService.showAlert( 'Rescan Failed. Order was not scanned previously!', AlertType.ERROR
      );
    }
  }


  //for the case the user lost the link to its pdf (user receives an email with the link  - PDF will not be modified)
  resendEmailPdfLink(order: OrderResponseDto) {
    this.loadingService.isLoading.next(true);
    this.subscriptions.push(
      this.orderService.resendEmailLinkPdf({ orderId: order.id }).subscribe({
        next: (res) => {
          this.loadingService.isLoading.next(false);
          this.alertService.showAlert('Email link sent successfully', AlertType.SUCCESS );
        },
        error: (err) => {
          this.loadingService.isLoading.next(false);
          this.alertService.showAlert('Failed to send email link. Please try again later or contact the administrator.', AlertType.ERROR );
        }
      })
    )
  }

  ngOnDestroy(): void {
    this.subscriptions.forEach((sub) => sub.unsubscribe());
  }

}