import { HttpClient } from '@angular/common/http';
import { AfterViewInit, Component, OnDestroy, OnInit, ViewChild } from '@angular/core';
import { MatDialog } from '@angular/material/dialog';
import { Router } from '@angular/router';
import { AlertType } from 'src/app/alert/alert-type.enum';
import { AlertService } from 'src/app/services/alert.service';
import { LoadingService } from 'src/app/services/loading.service';
import { GetChairsAvailableOrders$Params } from 'src/openapi/generated/fn/order/get-chairs-available-orders';
import {
  OrderResponseDto,
} from 'src/openapi/generated/models';
import {
  AdminOrderService,
  OrderService,
} from 'src/openapi/generated/services';
import { MoveOrderDialogComponent } from '../move-order-dialog/move-order-dialog.component';
import { MoveOrder$Params } from 'src/openapi/generated/fn/admin-order/move-order';
import { catchError, map, merge, Observable, of, startWith, Subscription, switchMap } from 'rxjs';
import { HandleRejectOrder$Params } from 'src/openapi/generated/fn/order/handle-reject-order';
import { OrderRejectDialogComponent } from '../order-reject-dialog/order-reject-dialog.component';
import { MatPaginator } from '@angular/material/paginator';
import { MatSort, SortDirection } from '@angular/material/sort';
import { MatTableDataSource } from '@angular/material/table';
import { ConfirmationDialogComponent } from 'src/app/confirmation-dialog/confirmation-dialog.component';
import { HandlePostponeOrder$Params } from 'src/openapi/generated/fn/order/handle-postpone-order';
import { TokenService } from 'src/app/services/token.service';

@Component({
  selector: 'app-chairs-available-orders',
  templateUrl: './chairs-available-orders.component.html',
  styleUrls: ['./chairs-available-orders.component.css'],
})
export class ChairsAvailableOrdersComponent implements OnDestroy, AfterViewInit {

  displayedColumns: string[] = ['id', 'dueDate', 'clientDeliveryEmail', 'signature', 'author', 'title', 'bvNr', 'isbnissn', 'actions'];
  dataSource: MatTableDataSource<OrderResponseDto> = new MatTableDataSource<OrderResponseDto>();  
  resultsLength = 0;
  isAdmin$!: Observable<boolean>;
  @ViewChild(MatPaginator) paginator: MatPaginator;
  @ViewChild(MatSort) sort: MatSort;
  subscriptions: Subscription[] = [];


  constructor(
    private loadingService: LoadingService,
    private alertService: AlertService,
    private orderService: OrderService,
    private adminOrderService: AdminOrderService,
    private router: Router,
    public dialog: MatDialog,
    private tokenService: TokenService
  ) {
    // this.subscriptions.push(
    //   this.orderService.getChairsAvailableOrders({pageNo: 0, pageSize: 5, sortBy: 'dueDate', sortDir: 'asc'}).subscribe(data =>
    //    {
    //     this.resultsLength = 0; // data.totalElements || 0;
    //     this.dataSource.data = []; //data.content || [];
    //    }
    //   )
    // );
  }

  ngAfterViewInit() {
    this.isAdmin$ = this.tokenService.userAuthority$.pipe(
      map(authority => authority == 'ROLE_ADMIN')
    );

    this.tokenService.setEditView(true);
    //if table is not created bec has no orders, skip checking pagination and sorting
    if (this.sort && this.paginator) {
      // If the user changes the sort order, reset back to the first page.
      this.sort.sortChange.subscribe(() => (this.paginator.pageIndex = 0));
      this.subscriptions.push(
        //check if sort or pagination observable values changes to update the table
        merge(this.sort.sortChange, this.paginator.page)
          .pipe(
            startWith({}),
            switchMap(() => {
              this.loadingService.isLoading.next(true);
              return this.findChairsAvailableOrders().pipe(catchError(() => { this.alertService.showAlert('Could not load orders', AlertType.ERROR); return of(null); }));
            }),
            map(data => {
              // Flip flag to show that loading has finished.
              this.loadingService.isLoading.next(false);
              if (data === null) {
                return [];
              }
              // Only refresh the result length if there is new data. 
              this.resultsLength = data.totalElements;
              return data.content;
            }),
          )
          .subscribe(data => { this.dataSource.data =  data })
      );
    }
  }

  private resetPaginationAndSorting(): void {
    this.paginator.pageIndex = 0;
    this.paginator.pageSize = 5;
    this.sort.active = 'dueDate';
    this.sort.direction = 'asc';
  }

  applyFilter(event: Event) {
    const filterValue = (event.target as HTMLInputElement).value;
    this.dataSource.filter = filterValue.trim().toLowerCase();
    if (this.dataSource.paginator) {
      this.dataSource.paginator.firstPage();
    }
  }

  private findChairsAvailableOrders() : Observable<any> {
    let params: GetChairsAvailableOrders$Params = {
      pageNo: this.paginator.pageIndex,
      pageSize: this.paginator.pageSize,
      sortBy: this.sort.active === 'signature' ? 'document.signature' : this.sort.active,
      sortDir: this.sort.direction,
    };
    return this.orderService.getChairsAvailableOrders(params);
  }

  viewOrder(order: OrderResponseDto) {
    this.router.navigate(['orders', 'manage', order.id]);
  }

  moveOrderToOtherChair(order: OrderResponseDto) {
    const dialogRef = this.dialog.open(MoveOrderDialogComponent, {
      width: '300px',
      data: {},
    });
    this.subscriptions.push(
      dialogRef.afterClosed().subscribe((result) => {
        if (result) {
          this.loadingService.isLoading.next(true);
          this.adminOrderService.moveOrder({ id: order.id, chair: result.selectedChair }).subscribe({
              next: (res) => {
                //this.findChairsAvailableOrders();
                this.resetPaginationAndSorting();
                this.alertService.showAlert('Order moved succesfully', AlertType.SUCCESS);
                this.subscriptions.push(
                this.findChairsAvailableOrders().subscribe(
                      res => {
                        this.loadingService.isLoading.next(false);
                        if (res !== null) {
                          this.dataSource.data = res.content;
                          this.resultsLength = res.totalElements;
                        }
                      },
                      err => {
                        this.loadingService.isLoading.next(false);
                        this.alertService.showAlert('Something went wrong while fetching the data. Please try again later.', AlertType.ERROR);
                      }
                    )
                  )
              },
              error: (err) => {
                this.loadingService.isLoading.next(false);
                this.alertService.showAlert('Something went wrong while moving the order. Please try again later.', AlertType.ERROR);
              },
            });
        }
      })
    );
  }

  extendDueDate(order: OrderResponseDto) {
      const dialogRef = this.dialog.open(ConfirmationDialogComponent);   
      dialogRef.afterClosed().subscribe(result => {
        if (result) {
          this.loadingService.isLoading.next(true);
          let params: HandlePostponeOrder$Params = {
            orderId: order.id
          };
            this.orderService.handlePostponeOrder(params).subscribe({
              next: (res) => {
                this.resetPaginationAndSorting();
                this.loadingService.isLoading.next(false);
                this.alertService.showAlert('Order postponed!', AlertType.SUCCESS );
                this.subscriptions.push(
                  this.findChairsAvailableOrders().subscribe(
                        res => {
                          this.loadingService.isLoading.next(false);
                          if (res !== null) {
                            this.dataSource.data = res.content;
                            this.resultsLength = res.totalElements;
                          }
                        },
                        err => {
                          this.loadingService.isLoading.next(false);
                          this.alertService.showAlert('Something went wrong while fetching the data. Please try again later.', AlertType.ERROR);
                        }
                      )
                    )
              },
              error: (err) => {
                this.alertService.showAlert('Failed to postpone the order. Please try again later.', AlertType.ERROR );
                this.loadingService.isLoading.next(false);
              }
            });
          } 
       }
     );  
  }

  scanOrder(order: OrderResponseDto) {
    this.router.navigate(['orders', 'scan', order.id]);
  }

  rejectOrder(order: OrderResponseDto) {
    let reasonRejectionModal = '';
    //open modal and let worker select reason of rejection
    const dialogRef = this.dialog.open(OrderRejectDialogComponent, {
      width: '500px',
      data: {},
    });
    this.subscriptions.push(
      dialogRef.afterClosed().subscribe((result) => {
        if (result.reason) {
          // Handle the rejection reason
          if (result.reason === 'Other Reason') {
            reasonRejectionModal = result.otherReason;
          } else {
            reasonRejectionModal = result.reason;
          }
          this.loadingService.isLoading.next(true);
          const param: HandleRejectOrder$Params = {
            orderId: order.id,
            body: { reasonRejection: reasonRejectionModal },
          };
          this.orderService.handleRejectOrder(param).subscribe({
            next: (res) => {
              this.loadingService.isLoading.next(false);
              this.alertService.showAlert('Order rejected successfully', AlertType.SUCCESS);
              this.resetPaginationAndSorting();
              this.subscriptions.push(
              this.findChairsAvailableOrders().subscribe(
                    res => {
                      this.loadingService.isLoading.next(false);
                      if (res !== null) {
                        this.dataSource.data = res.content;
                        this.resultsLength = res.totalElements;
                      }
                    },
                    err => {
                      this.loadingService.isLoading.next(false);
                      this.alertService.showAlert('Something went wrong while fetching the data. Please try again later.', AlertType.ERROR);
                    }
                  )
                )
            },
            error: (err) => {
              this.loadingService.isLoading.next(false);
              this.alertService.showAlert('Something went wrong. Please try again later.', AlertType.ERROR);
            },
          });
        }
      })
    );
  }

  isToday(date: Date | string): boolean {
    return new Date().toDateString() === new Date(date).toDateString();
  }

  ngOnDestroy(): void {
    this.subscriptions.forEach((sub) => sub.unsubscribe());
  }


}
