import { ActivatedRoute, ActivatedRouteSnapshot, Router } from '@angular/router';
import { Component, Input, OnDestroy, OnInit } from '@angular/core';
import { Location } from '@angular/common';
import cloneDeep from 'lodash/cloneDeep';
import { OrderRequestDto, OrderResponseDto } from 'src/openapi/generated/models';
import { FormBuilder, FormControl, FormGroup, Validators } from '@angular/forms';
import { TokenService } from 'src/app/services/token.service';
import { AdminOrderService, ManagementService, OrderService } from 'src/openapi/generated/services';
import { Subscription } from 'rxjs';
// import { UpdateAnyOrdersMetadata$Params } from 'src/openapi/generated/fn/admin-order/update-any-orders-metadata';
import { AlertService } from 'src/app/services/alert.service';
import { LoadingService } from 'src/app/services/loading.service';
import { AlertType } from 'src/app/alert/alert-type.enum';
import { UpdateOrderData$Params } from 'src/openapi/generated/fn/order/update-order-data';

@Component({
  selector: 'app-manage-order',
  templateUrl: './manage-order.component.html',
  styleUrls: ['./manage-order.component.css']
})
export class ManageOrderComponent implements OnInit, OnDestroy {

  editMode: boolean = false;
  orderForm: FormGroup;
  order: OrderResponseDto = null;
  role = '';
  showEdit: boolean = true;
  subscriptions: Subscription[] = [];

  constructor(
    private loadingService: LoadingService,
    private alertService: AlertService,
    private orderService: OrderService,
    private tokenService: TokenService, 
    private managementService: ManagementService,
    private route: ActivatedRoute,
    private router: Router,
    private fb: FormBuilder,
  ) { }

  ngOnInit(): void {
    this.showEdit = this.tokenService.getEditView();
    this.route.data.subscribe(data => {
      this.order = data['resolvedOrderDetails'];
      this.initRole();
      this.initForm();
    });
    //this.orderForm.patchValue(this.order);
  }

  initRole() {
    this.subscriptions.push(
      this.tokenService.userAuthority$.subscribe(role => {
        this.role = role;
      })
    );
  }

  private initForm() {
    this.orderForm = this.fb.group({
      authorBook: new FormControl({value: this.order.authorBook, disabled: true}),
      authorArticle: new FormControl({value: this.order.authorArticle, disabled: true}),
      bvNr: new FormControl({value: this.order.bvNr, disabled: true },  [Validators.required]),
      clientDeliveryEmail: new FormControl({value: this.order.clientDeliveryEmail, disabled: true}),
      clientDeliveryName: new FormControl({value: this.order.clientDeliveryEmail, disabled: true}),
      isbn: new FormControl({value: this.order.isbn, disabled: true}),
      issn: new FormControl({value: this.order.issn, disabled: true}),
      issue: new FormControl({value: this.order.issue, disabled: true}),
      issueArticle: new FormControl({value: this.order.issueArticle, disabled: true}),
      pages: new FormControl({value: this.order.pages, disabled: true}, [Validators.required]),
      placePublication: new FormControl({value: this.order.placePublication, disabled: true}),
      publicationDate: new FormControl({value: this.order.publicationDate, disabled: true}),
      publisher: new FormControl({value: this.order.publisher, disabled: true}),
      signature: new FormControl({value: this.order.signature, disabled: true}, [Validators.required]),
      scanComment: new FormControl({value: this.order.scanComment, disabled: true}),
      titleBook: new FormControl({value: this.order.titleBook, disabled: true}),
      titleArticle: new FormControl({value: this.order.titleArticle, disabled: true}),
      type: new FormControl({ value: this.order.type, disabled: true }, [Validators.required]), 
      //we allow only document related fields to be updated
      //id: [{ value: this.order.id, disabled: !this.editMode }], // Set as disabled if editing an existing order
      //transactionGroupQualifier: new FormControl({value: this.order.transactionGroupQualifier, disabled: true}),
      //userGroup: [{ value: this.order.userGroup, disabled: !this.editMode }, Validators.required],
      //state: [{ value: this.order.state, disabled: !this.editMode }, Validators.required],
      //doneDate: [{ value: this.order.dueDate, disabled: !this.editMode }],
      //dueDate: [{ value: this.order.dueDate, disabled: !this.editMode }, Validators.required],
    });
  }

  onEdit() {
    this.editMode = true;
    const disabledFields = new Set(['clientDeliveryEmail', 'clientDeliveryName', 'scanComment']);
    Object.entries(this.orderForm.controls).forEach(([controlName, control]) => {
      disabledFields.has(controlName) ? control.disable() : control.enable();
    });
  }

  onCancel(){ 
    this.editMode = false;
    this.orderForm.reset(this.order);
    Object.keys(this.orderForm.controls).forEach(controlName => {
      this.orderForm.get(controlName).disable();
    });
  }

  onSubmit() {
    if(this.orderForm.valid) {
      this.loadingService.isLoading.next(true);
      const updatedOrderValues: OrderRequestDto = {
        bvNr: this.orderForm.get('bvNr').value,
        pages: this.orderForm.get('pages').value,
        signature: this.orderForm.get('signature').value,
        ...Object.fromEntries(
          Object.entries(this.orderForm.value).filter(
            ([controlName]) => !['clientDeliveryEmail', 'clientDeliveryName', 'scanComment'].includes(controlName)
          )
        ),
      };
      const params: UpdateOrderData$Params = {
        orderId: this.order.id,
        body: updatedOrderValues
      }
      this.orderService.updateOrderData(params).subscribe(order =>{
        this.loadingService.isLoading.next(false);
        this.alertService.showAlert('Order has been updated successfully', AlertType.SUCCESS );
        this.router.navigate(['/orders']);
      }, err => {
        this.alertService.showAlert('Failed to save the order. Please try again later.', AlertType.ERROR );
        this.loadingService.isLoading.next(false);
        this.onCancel();
        console.log(err);
      })
    }
  }


  goBack() {
    this.router.navigate(['/orders/available']);
  }

  ngOnDestroy() {
    this.subscriptions.forEach(sub => sub.unsubscribe());
  }

}
