import { OrderService } from 'src/openapi/generated/services';
import { Component, Input, OnDestroy, OnInit } from '@angular/core';
import jsPDF from 'jspdf';
import { PDFDocumentProxy } from 'ng2-pdf-viewer';
import { catchError, map, Observable, of, Subscription, switchMap, tap } from 'rxjs';
import { OrderResponseDto } from 'src/openapi/generated/models/order-response-dto';
import { MergeOrderPdf$Params } from 'src/openapi/generated/fn/order/merge-order-pdf';
import { AlertService } from 'src/app/services/alert.service';
import { LoadingService } from 'src/app/services/loading.service';
import { AlertType } from 'src/app/alert/alert-type.enum';
import { ActivatedRoute, Router } from '@angular/router';
import { ScanRequestDto } from 'src/openapi/generated/models';
import { PDFDocument } from 'pdf-lib';
import { ConfirmationDialogComponent } from 'src/app/confirmation-dialog/confirmation-dialog.component';
import { CanComponentDeactivate } from 'src/app/services/can-component-deactivate';
import { MatDialog } from '@angular/material/dialog';
import { Location } from '@angular/common';
import { NavigationService } from 'src/app/services/navigation.service';

@Component({
  selector: 'app-merge-pdf',
  templateUrl: './merge-pdf.component.html',
  styleUrls: ['./merge-pdf.component.css']
})
export class MergePdfComponent implements CanComponentDeactivate, OnInit, OnDestroy {
  
  @Input() order: OrderResponseDto;
  //to display 1 page of a pdf document per time
  totalPagesPdf!: number;
  page: number = 1;
  isPdfLoaded: boolean = false;
  isPdfError: boolean = false;
  isNow$: Observable<boolean>;
  scannedPdf: Uint8Array | null = null;
  uploadedPdf: File | null = null;
  scannedPdfUrl: string;
  scanComment: string = '';
  confirmNavigation = false;
  subs: Subscription[] = [];
  

  constructor(
    private route: ActivatedRoute,
    private router: Router,
    private location: Location,
    private orderService: OrderService,
    private alertService: AlertService,
    private loadingService: LoadingService,
    private dialog: MatDialog,
    private navigationService: NavigationService,
  ) {}


  ngOnInit(): void {
    this.subs.push(
    //get order from route
      this.route.data.pipe(
        tap(data => this.order = data['resolvedOrderDetails']),
        //the original PDF was renamed to ...OLD.pdf in the backend, so we show the old one and allow it to be merge to other document
        switchMap(() => this.orderService.getOrderPdf({ orderId: this.order.id, obsolete: true }))
      ).subscribe(
        //get scanned pdf to display
        pdf => {
          const reader = new FileReader();
          reader.onload = () => {
            const arrayBuffer = reader.result as ArrayBuffer;
            this.scannedPdf = new Uint8Array(arrayBuffer);
            const blob = new Blob([this.scannedPdf], { type: 'application/pdf' });
            this.scannedPdfUrl = URL.createObjectURL(blob);
          };
          reader.readAsArrayBuffer(pdf);
        }
      ));
  }


  onFileSelected(event: any) {
    let fileList: FileList = event.target.files;
    //if no file provided, return
    if (fileList.length < 1) {
      return;
    }
    this.uploadedPdf = fileList[0];
    if (this.uploadedPdf.type !== 'application/pdf') {
      this.alertService.showAlert("Selected file is not a PDF!", AlertType.ERROR);
      this.uploadedPdf = null;
    }
    if (this.uploadedPdf.size > 100 * 1024 * 1024) { // 100MB limit
      this.alertService.showAlert("File size exceeds the limit of 100MB!", AlertType.ERROR);
      this.uploadedPdf = null;
    }
  }


  async mergePdfs() {
    if (this.uploadedPdf && this.scannedPdf) {
      this.loadingService.isLoading.next(true);
      const uploadedPdfBuffer = await this.uploadedPdf.arrayBuffer();
      // Create pdf-lib PDFDocument instances
      const scannedPdfDoc = await PDFDocument.load(this.scannedPdf.buffer);
      const uploadedPdfDoc = await PDFDocument.load(uploadedPdfBuffer);
      // Create a new PDFDocument for the merged PDF
      const mergedPdfDoc = await PDFDocument.create();
      // Copy pages from scanned PDF into the new PDF
      const scannedPages = await mergedPdfDoc.copyPages(scannedPdfDoc, scannedPdfDoc.getPageIndices());
      scannedPages.forEach(page => mergedPdfDoc.addPage(page));
      // Copy pages from uploaded PDF into the new PDF
      const uploadedPages = await mergedPdfDoc.copyPages(uploadedPdfDoc, uploadedPdfDoc.getPageIndices());
      uploadedPages.forEach(page => mergedPdfDoc.addPage(page));
      // Convert the merged PDF to a Blob
      const mergedPdfBytes = await mergedPdfDoc.save();
      const mergedPdfBlob = new Blob([mergedPdfBytes], { type: 'application/pdf' });
 //for debug, to see the merged PDF in new tab     
 //     const mergedPdfBlobUrl = URL.createObjectURL(mergedPdfBlob);
 //     window.open(mergedPdfBlobUrl, '_blank');
      // pass the merged Blob to the FormData
      const formData = new FormData();
      formData.append('file', mergedPdfBlob, 'merged.pdf');
      // Add scanRequest as JSON Blob too the FormData
      const scanRequest: ScanRequestDto = {
        id: this.order.id,
        scanComment: this.scanComment
      };
      formData.append('scanRequest', new Blob([JSON.stringify(scanRequest)], { type: 'application/json' }));
// for debug, to see in console what is in the formData
//     formData.forEach((value, key) => {
//       console.log(`${key}:`, value);
//     });
      // Send the merged PDF to backend for saving
      const params: MergeOrderPdf$Params = {
        body: formData
      };  
      this.subs.push(
        this.orderService.mergeOrderPdf(params)
          .subscribe(
          res => {
            this.loadingService.isLoading.next(false);
            this.alertService.showAlert("PDF merged successfully", AlertType.SUCCESS);
            this.confirmNavigation = true;
            this.goBack();
          },
          error => {
            this.loadingService.isLoading.next(false);
            this.alertService.showAlert("PDF merge failed! Retry or contact the administrator.", AlertType.ERROR);
            this.confirmNavigation = true;
            this.goBack();
          }
        )
      );
    } else {
      this.alertService.showAlert("Please select a PDF to merge", AlertType.INFO);
    }
  }


  goBack() {
    // if (window.history.length > 1) {
    //   this.location.back();
    // } else {
      this.router.navigate(['/orders/archived']);
    // }
  }
 
 
  canDeactivate(): Observable<boolean> {
    //if logout pending, cancel rescan andrename file from .pdfOLD to .pdf (s.t. the order has the original file available if we logout and did not replace with new file)
    if(this.navigationService.getLogoutPending()) {
      return this.orderService.handleCancelRescanFile({ orderId: this.order.id }).pipe( 
        catchError(() => {
          this.alertService.showAlert("Something went wrong while cancelling. Order can be found in 'Inprogress Orders'.", AlertType.ERROR);
          return of(true); // Allow navigation if cancelRescan call fails
        }),
        map(() => true) // Final response to allow navigation
      );
    }
    //if modal  already confirmed => skip displaying confirmation modal and allow navigation
    if(this.confirmNavigation) {
      return of(true);
    }

     const dialogRef = this.dialog.open(ConfirmationDialogComponent, {
       data: { message: 'Are you sure you want to leave? Unsaved changes may be lost.', confirmButtonText: 'Yes', cancelButtonText: 'No', }
     });
      // Convert the dialog and service calls to Observables and use `switchMap` to handle them sequentially
     return dialogRef.afterClosed().pipe(
       switchMap(result => {
         if (!result) {
           return of(false);  // User canceled the dialog, so stay on the page
         }
         // Set flag to allow navigation post-confirmation
         this.confirmNavigation = true;
         // If user confirmed, rename the file by calling the service
         this.loadingService.isLoading.next(true);
         return this.orderService.handleCancelRescanFile({ orderId: this.order.id }).pipe(
           tap(() => {
             this.loadingService.isLoading.next(false);
             this.alertService.showAlert("Rescan cancelled successfully", AlertType.SUCCESS);
           }),
           catchError(() => {
             this.loadingService.isLoading.next(false);
             this.alertService.showAlert("Something went wrong while cancelling. Order can be found in 'Inprogress Orders'.", AlertType.ERROR);
             return of(true); // Allow navigation if cancelRescan call fails
           }),
           map(() => true) // Final response to allow navigation
         );
       })
     );
   }

  
  //to display 1 page of a pdf document per time
  callBackFn(pdf: PDFDocumentProxy){
    this.totalPagesPdf = pdf.numPages;
    this.isPdfLoaded = true;
    this.isPdfError = false;
  }

  onPdfError(error: any) {
    console.log(error);
    this.isPdfError = true;
  }

  nextPdfPage() {
    this.page++;
  }

  prevPdfPage() {
    this.page--;
  }

  goFirstPage() {
    this.page = 1;
  }

  goLastPage() {
    this.page = this.totalPagesPdf;
  }

  privatearrayBufferToBase64(buffer: ArrayBuffer): string {
    let binary = '';
    const bytes = new Uint8Array(buffer);
    const len = bytes.byteLength;
    for (let i = 0; i < len; i++) {
      binary += String.fromCharCode(bytes[i]);
    }
    return window.btoa(binary);
  }
  
  ngOnDestroy(): void {
    this.subs.forEach(sub => sub.unsubscribe());
  }
}
