import { Location } from '@angular/common';
import { Component, ElementRef, OnDestroy, OnInit, ViewChild } from '@angular/core';
import { FormGroup, FormControl, Validators, FormBuilder } from '@angular/forms';
import { MatDialog } from '@angular/material/dialog';
import { ActivatedRoute, Router } from '@angular/router';
import { jsPDF } from 'jspdf';
import { error } from 'pdf-lib';
import { catchError, map, Observable, of, switchMap, take, tap } from 'rxjs';
import { Subscription } from 'rxjs/internal/Subscription';
import { AlertType } from 'src/app/alert/alert-type.enum';
import { ConfirmationDialogComponent } from 'src/app/confirmation-dialog/confirmation-dialog.component';
import { AlertService } from 'src/app/services/alert.service';
import { CanComponentDeactivate } from 'src/app/services/can-component-deactivate';
import { LoadingService } from 'src/app/services/loading.service';
import { NavigationService } from 'src/app/services/navigation.service';
import { TokenService } from 'src/app/services/token.service';
import { HandleScanOrder$Params } from 'src/openapi/generated/fn/order/handle-scan-order';
import { OrderResponseDto } from 'src/openapi/generated/models';
import { ManagementService, OrderService } from 'src/openapi/generated/services';

@Component({
  selector: 'app-order-scan-view',
  templateUrl: './order-scan-view.component.html',
  styleUrls: ['./order-scan-view.component.css']
})
export class OrderScanViewComponent implements CanComponentDeactivate, OnInit, OnDestroy {

  @ViewChild('qrcode', { static: false }) qrcode: ElementRef;
  orderForm: FormGroup;
  order: OrderResponseDto = null;
  scanComment: string = '';
  role = 'ROLE_USER';
  showOrderDetails: boolean = false; // Toggle to switch views
  private confirmNavigation = false;
  subscriptions: Subscription[] = [];

  constructor(
    private navigationService: NavigationService,
    private loadingService: LoadingService,
    private alertService: AlertService,
    private orderService: OrderService,
    private tokenService: TokenService, 
    private managementService: ManagementService,
    private route: ActivatedRoute,
    private router: Router,
    private location: Location,
    private fb: FormBuilder,
    public dialog: MatDialog,
  ) { }

  ngOnInit(): void {
    this.subscriptions.push(
      this.route.data.subscribe(data => {
        this.order = data['resolvedOrderDetails'];
        this.initRole()
        this.initForm();
      })
    );
    
    //if order was in error and sent to 'INPROGRESS' after scan/rescan new file, we dont want to lose the scan so we provide option to use it
    if (this.order.error && this.order.state === 'INPROGRESS') {
      this.subscriptions.push(
        this.orderService.getOrderPdf({ orderId: this.order.id, obsolete: false })
        .pipe(take(1))
        .subscribe(
          data => {
            // File exists, show the confirmation dialog
            const dialogRef = this.dialog.open(ConfirmationDialogComponent, {
              data: { message: "The order has a scan from a previous process. Click 'Confirm' to use this scan or 'Cancel' to scan a new PDF." }
            });
    
            dialogRef.afterClosed().subscribe(result => {
              if (result) {
                // User chose to use the existing scan
                this.loadingService.isLoading.next(true);
                const params: HandleScanOrder$Params = { body: { id: this.order.id, scanComment: '' } };
                this.orderService.handleScanOrder(params).subscribe({
                  next: (res) => {
                    this.loadingService.isLoading.next(false);
                    this.confirmNavigation = true;
                    this.alertService.showAlert('Order scanned!', AlertType.SUCCESS);
                    this.goBack();
                  },
                  error: (err) => {
                    this.loadingService.isLoading.next(false);
                    this.confirmNavigation = true;
                    this.alertService.showAlert('Scan Failed. Please try again later.', AlertType.ERROR);
                    this.goBack();
                  }
                });
              }
            });
          }
        )
      );
    }
    
  }

  initRole() {
    this.subscriptions.push(
      this.tokenService.userAuthority$.subscribe(role => {
        this.role = role;
      })
    );
  }

  private initForm() {
    this.orderForm = this.fb.group({
      authorBook: new FormControl({value: this.order.authorBook, disabled: true}),
      authorArticle: new FormControl({value: this.order.authorArticle, disabled: true}),
      bvNr: new FormControl({value: this.order.bvNr, disabled: true },  [Validators.required]),
      clientDeliveryEmail: new FormControl({value: this.order.clientDeliveryEmail, disabled: true},  [Validators.required, Validators.email]),
      clientDeliveryName: new FormControl({value: this.order.clientDeliveryEmail, disabled: true}, [Validators.required]),
      isbn: new FormControl({value: this.order.isbn, disabled: true}),
      issn: new FormControl({value: this.order.issn, disabled: true}),
      issue: new FormControl({value: this.order.issue, disabled: true}),
      issueArticle: new FormControl({value: this.order.issueArticle, disabled: true}),
      pages: new FormControl({value: this.order.pages, disabled: true}, [Validators.required]),
      placePublication: new FormControl({value: this.order.placePublication, disabled: true}),
      publicationDate: new FormControl({value: this.order.publicationDate, disabled: true}),
      publisher: new FormControl({value: this.order.publisher, disabled: true}),
      signature: new FormControl({value: this.order.signature, disabled: true}, [Validators.required]),
      scanComment: new FormControl({value: this.order.scanComment, disabled: true}),
      titleBook: new FormControl({value: this.order.titleBook, disabled: true}),
      titleArticle: new FormControl({value: this.order.titleArticle, disabled: true}),
      type: new FormControl({ value: this.order.type, disabled: true }, [Validators.required]), 
      //we allow only document related fields to be updated
      //id: [{ value: this.order.id, disabled: !this.editMode }], // Set as disabled if editing an existing order
      //scanPath: [{ value: this.order.publisher, disabled: !this.editMode }], 
      //transactionGroupQualifier: new FormControl({value: this.order.transactionGroupQualifier, disabled: true}),
      //userGroup: [{ value: this.order.userGroup, disabled: !this.editMode }, Validators.required],
      //state: [{ value: this.order.state, disabled: !this.editMode }, Validators.required],
      //doneDate: [{ value: this.order.dueDate, disabled: !this.editMode }],
      //dueDate: [{ value: this.order.dueDate, disabled: !this.editMode }, Validators.required],
    });
  }


  onDone() {
    const dialogRef = this.dialog.open(ConfirmationDialogComponent, {
      data: { message: "Have you completed scanning all the pages and wish to mark the order as finished?" }
    });
    dialogRef.afterClosed().subscribe(result => {
      if (result) {
        this.loadingService.isLoading.next(true);
        const params: HandleScanOrder$Params = { body: {
          id: this.order.id,
          scanComment: this.scanComment
        }}
        this.orderService.handleScanOrder(params).subscribe({
          next: (res) => {
            this.loadingService.isLoading.next(false);
            this.confirmNavigation = true;
            this.alertService.showAlert('Order scanned!', AlertType.SUCCESS );
            this.goBack();
          },
          error: (err) => {
            this.loadingService.isLoading.next(false);
            this.confirmNavigation = true;
            this.alertService.showAlert('Scan Failed. Please try again later.', AlertType.ERROR );
            this.goBack();
          }
        });
     }
   });  
  }

  downloadQrCode() {
    let yposition= 85;
    const doc = new jsPDF();
    const qrCodeElement = this.qrcode.nativeElement.querySelector('canvas');
    const imgData = qrCodeElement.toDataURL('image/png');
    doc.addImage(imgData, 'PNG', 60, 5, 80, 80);
    // Add order details to PDF
    doc.setFontSize(12);
    if(this.order.titleBook) { yposition = yposition + 10; doc.text(`Title Book: ${this.order.titleBook}`, 10, yposition); }
    if(this.order.authorBook) { yposition = yposition + 10; doc.text(`Author Book: ${this.order.authorBook}`, 10, yposition); }
    if(this.order.titleArticle) { yposition = yposition + 10; doc.text(`Title Article: ${this.order.titleArticle}`, 10, yposition); }
    if(this.order.authorArticle) { yposition = yposition + 10; doc.text(`Author Article: ${this.order.authorArticle}`, 10, yposition); }
    if(this.order.isbn) { yposition = yposition + 10; doc.text(`ISBN: ${this.order.isbn}`, 10, yposition); }
    if(this.order.issn) { yposition = yposition + 10; doc.text(`ISSN: ${this.order.issn}`, 10, yposition); }
    if(this.order.issue) { yposition = yposition + 10; doc.text(`Issue: ${this.order.issue}`, 10, yposition); }
    if(this.order.issueArticle) { yposition = yposition + 10; doc.text(`Issue Article: ${this.order.issueArticle}`, 10, yposition); }
    yposition = yposition + 10;
    doc.text(`BV Nr: ${this.order.bvNr}`, 10, yposition);
    yposition = yposition + 10;
    doc.text(`Signature: ${this.order.signature}`, 10, yposition);
    yposition = yposition + 10;
    doc.text(`Pages: ${this.order.pages}`, 10, yposition);
    yposition = yposition + 10;
    doc.text(`Place Publication: ${this.order.placePublication}`, 10, yposition);
    yposition = yposition + 10;
    doc.text(`Date Publication: ${this.order.publicationDate}`, 10, yposition);
    yposition = yposition + 10;
    doc.text(`Publisher: ${this.order.publisher}`, 10, yposition);
    yposition = yposition + 10;
    doc.text(`Type: ${this.order.type}`, 10, yposition);
    yposition = yposition + 10;
    doc.text(`Client Delivery Email: ${this.order.clientDeliveryEmail}`, 10, yposition);
    yposition = yposition + 10;
    doc.text(`Client Delivery Name: ${this.order.clientDeliveryName}`, 10, yposition );
    yposition = yposition + 10;
    doc.text(`Due Date: ${this.order.dueDate}`, 10, yposition);
    //doc.text(`Worker ID: ${this.order.faudokWorkerId || 'Unassigned'}`, 10, 80);
    // Save PDF
    doc.save('QRCode_Order_Details.pdf');
  }

  goBack() {
    // if (window.history.length > 1) {
    //   this.location.back();
    // } else {
      this.router.navigate(['/orders/available']);
    // }
  }


  canDeactivate(): Observable<boolean> {
     //if not scanned yet || modal  already confirmed => skip displaying confirmation modal and allow navigation 
     if(!this.order.scanned || this.confirmNavigation) {
      return of(true);
    }
    //if logout pending, cancel rescan and rename file from .pdfOLD to .pdf (s.t. the order has the original file available if we logout and did not replace with new file)
    if(this.navigationService.getLogoutPending()) {
       return this.orderService.handleCancelRescanFile({ orderId: this.order.id }).pipe( 
        catchError(() => {
          this.alertService.showAlert("Something went wrong while cancelling. Order can be found in 'Inprogress Orders'.", AlertType.ERROR);
          return of(true); // Allow navigation if cancelRescan call fails
        }),
        map(() => true) // Final response to allow navigation
      );
    }

    const dialogRef = this.dialog.open(ConfirmationDialogComponent, {
      data: { message: 'Are you sure you want to leave? Unsaved changes may be lost.', confirmButtonText: 'Yes', cancelButtonText: 'No', },
    });
     // Convert the dialog and service calls to Observables and use `switchMap` to handle them sequentially
    return dialogRef.afterClosed().pipe(
      switchMap(result => {
        // User canceled the dialog, so stay on the page
        if (!result) { return of(false); }
        // Set flag to allow navigation post-confirmation
        this.confirmNavigation = true;
        // If user confirmed, rename the file by calling the service
        this.loadingService.isLoading.next(true);
        return this.orderService.handleCancelRescanFile({ orderId: this.order.id }).pipe(
          tap(() => {
            this.loadingService.isLoading.next(false);
            this.alertService.showAlert("Rescan cancelled successfully", AlertType.SUCCESS);
          }),
          catchError(() => {
            this.loadingService.isLoading.next(false);
            this.alertService.showAlert("Something went wrong while cancelling. Order can be found in 'Inprogress Orders'.", AlertType.ERROR);
            return of(true); // Allow navigation if cancelRescan call fails
          }),
          map(() => true) // Final response to allow navigation
        );
      })
    );
  }


  ngOnDestroy() {
    this.subscriptions.forEach(sub => sub.unsubscribe());
  }


}
