import { AdminOrderService } from './../../../openapi/generated/services/admin-order.service';
import { AfterViewInit, Component, OnDestroy, OnInit, ViewChild } from '@angular/core';
import { MatPaginator } from '@angular/material/paginator';
import { MatSort } from '@angular/material/sort';
import { MatTableDataSource } from '@angular/material/table';
import { Router } from '@angular/router';
import { catchError, map, merge, Observable, of, startWith, Subscription, switchMap } from 'rxjs';
import { AlertType } from 'src/app/alert/alert-type.enum';
import { AlertService } from 'src/app/services/alert.service';
import { LoadingService } from 'src/app/services/loading.service';
import { TokenService } from 'src/app/services/token.service';
import { FaudokResponsePageOrderResponseDto, OrderResponseDto } from 'src/openapi/generated/models';
import { OrderService } from 'src/openapi/generated/services';
import { ManagementService } from 'src/openapi/generated/services/management.service';

@Component({
  selector: 'app-search',
  templateUrl: './search.component.html',
  styleUrls: ['./search.component.css']
})
export class SearchComponent implements OnInit, OnDestroy{

  searchCriteria: string = 'id';
  searchKeyword: string = '%';
  role = '';
  searchPage: SearchPage  = {'pageNo': 0, 'pageSize': 5, 'sortBy': 'dueDate', 'sortDir': 'asc'};
  ordersPage: FaudokResponsePageOrderResponseDto;

  displayedColumns: string[] = ['id', 'dueDate', 'doneDate', 'clientDeliveryEmail', 'signature', 'authorArticle', 'authorBook', 'titleArticle', 'titleBook', 'bvNr', 'isbn', 'issn', 'actions'];

  displayedColumns: string[] = ['id', 'dueDate', 'doneDate', 'clientDeliveryEmail', 'signature', 'author', 'title', 'bvNr', 'isbnissn', 'actions'];
  data: MatTableDataSource<OrderResponseDto> ;
  resultsLength = 0;
  hasOrders: boolean = true;
  @ViewChild(MatPaginator) paginator: MatPaginator;
  @ViewChild(MatSort) sort: MatSort;
  subscriptions: Subscription[] = [];

  constructor(
    private loadingService: LoadingService,
    private alertService: AlertService,
    private router: Router,
    private tokenService: TokenService,
    private managementService: ManagementService,
    private orderService: OrderService,
    private adminOrderService: AdminOrderService,
  ) { 
    //by default, display all orders by id
    this.onSearch();
  }

  
  onSearch() {
    if (!this.searchKeyword.trim()) {
      return null; // Do nothing if searchKeyword is empty or whitespace
    }
    //if table is not created because it has no orders, skip checking pagination and sorting
    if (this.sort && this.paginator) {
      // If the user changes the sort order, reset back to the first page.
      this.sort?.sortChange.subscribe(() => (this.paginator.pageIndex = 0));
      this.subscriptions.push(
        //check if sort or pagination observable values changes to update the table
        merge(this.sort.sortChange, this.paginator.page)
          .pipe(
            startWith({}),
            switchMap(() => {
              this.loadingService.isLoading.next(true);
              this.updatePaginationAndSorting();
              return this.handleSearch().pipe(
                catchError(() =>
                  { 
                    this.loadingService.isLoading.next(false);
                    this.hasOrders = false; 
                    this.alertService.showAlert('Failed to search orders. Please try again later or contact the administrator.', AlertType.ERROR );
                    return of(null);
                  })
              );
            }),
            map(data => {
              this.loadingService.isLoading.next(false);
              if (data === null) {
                this.hasOrders = false;
                return [];
              }
              this.resultsLength = data.totalElements;
              data.totalElements !== 0 ? this.hasOrders = true : this.hasOrders = false;
              return data.content;
            }),
          )
          .subscribe(data => (this.data = new MatTableDataSource( data ) ))
      );
    } else {
      //if prev have no orders but now search is pressed, because we dont have initialized a table with pagination and sorting, reset pagination and fetch orders
      this.loadingService.isLoading.next(true);
      this.resetPaginationAndSorting();
      this.subscriptions.push(
        this.handleSearch().subscribe(data => {
          this.loadingService.isLoading.next(false);
          if (data === null) {
            this.hasOrders = false;
            this.data =  new MatTableDataSource([]);
          }
          this.resultsLength = data.totalElements;
          data.totalElements !== 0 ? this.hasOrders = true : this.hasOrders = false;
          this.data = new MatTableDataSource(data.content);
        },
        error => {
          this.loadingService.isLoading.next(false);
          this.resetPaginationAndSorting();
          this.hasOrders = false; 
          this.alertService.showAlert('Failed to search orders. Please try again later or contact the administrator.', AlertType.ERROR );              
        }
       )
      );
    }
  }

  private resetPaginationAndSorting(): void {
    this.searchPage.pageNo = 0;
    this.searchPage.pageSize = 5;
    this.searchPage.sortBy = 'dueDate';
    this.searchPage.sortDir = 'asc';
  }

  private updatePaginationAndSorting(): void {
    this.searchPage.pageNo = this.paginator.pageIndex;
    this.searchPage.pageSize = this.paginator.pageSize;
    this.searchPage.sortBy = this.sort.active === 'signature' ? 'document.signature' : this.sort.active;
    this.searchPage.sortDir = this.sort.direction;
  }

  handleSearch(): Observable<any> {
    // console.log(this.searchCriteria, this.searchKeyword);
    //if role is admin, allow the search in all orders, otherwise allow it only in the user's team
    if(this.searchCriteria == 'id' && this.searchKeyword) {
      if(this.role == 'ROLE_ADMIN') {
        return this.adminOrderService.getAnyOrderByIdContaining({'orderId': this.searchKeyword, ...this.searchPage});
      } else {
        return this.orderService.getChairsOrderByIdContaining({'orderId': this.searchKeyword, ...this.searchPage});
      }
    }
    if(this.searchCriteria == 'signature' && this.searchKeyword) {
      if(this.role == 'ROLE_ADMIN') {
        return this.adminOrderService.getAnyOrderBySignatureContaining({'orderSignature': this.searchKeyword, ...this.searchPage});
      } else  {
        return this.orderService.getChairsOrderBySignatureContaining({'orderSignature': this.searchKeyword, ...this.searchPage});
      }
    }
    if(this.searchCriteria == 'email' && this.searchKeyword) {
      if(this.role == 'ROLE_ADMIN') {
        return this.adminOrderService.getAnyOrdersByClientEmailContaining({'clientEmail': this.searchKeyword, ...this.searchPage});
      } else {
        return this.orderService.getChairsOrderByClientEmailContaining({'clientEmail': this.searchKeyword, ...this.searchPage});
      }
    }
    //otherwise return nothing
    return of(null);
  }

  viewOrder(order: OrderResponseDto) {
    this.router.navigate(['orders','archived' ,'view', order.id]);
  }

  ngOnInit(): void {
    this.subscriptions.push(
      this.tokenService.userAuthority$.subscribe(role => {
        this.role = role;
      })
    );
  }

  ngOnDestroy(): void {
    this.subscriptions.forEach(sub => sub.unsubscribe());
  }
}


interface SearchPage {
  pageNo: number;
  pageSize: number;
  sortBy: string;
  sortDir: string;
}