import { AdminOrderService } from './../../../openapi/generated/services/admin-order.service';
import { AfterViewChecked, AfterViewInit, Component, OnDestroy, OnInit, ViewChild } from '@angular/core';
import { MatPaginator, PageEvent } from '@angular/material/paginator';
import { MatSelectChange } from '@angular/material/select';
import { MatSort, Sort } from '@angular/material/sort';
import { MatTableDataSource } from '@angular/material/table';
import { Router } from '@angular/router';
import { catchError, debounceTime, map, merge, Observable, of, startWith, Subject, Subscription, switchMap } from 'rxjs';
import { AlertType } from 'src/app/alert/alert-type.enum';
import { AlertService } from 'src/app/services/alert.service';
import { LoadingService } from 'src/app/services/loading.service';
import { TokenService } from 'src/app/services/token.service';
import { FaudokResponsePageOrderResponseDto, OrderResponseDto } from 'src/openapi/generated/models';
import { OrderService } from 'src/openapi/generated/services';
import { ManagementService } from 'src/openapi/generated/services/management.service';


  interface SearchParams {
    [key: string]: string | number | boolean;
    pageNo: number;
    pageSize: number;
    sortBy: string;
    sortDir: string;
  }

@Component({
  selector: 'app-search',
  templateUrl: './search.component.html',
  styleUrls: ['./search.component.css']
})
export class SearchComponent implements AfterViewInit, OnInit, OnDestroy {
  //options for search by
  searchOptions = [
    { value: 'orderId', viewValue: 'Order ID' },
    { value: 'orderSignature', viewValue: 'Shelf Number' },
    { value: 'clientEmail', viewValue: 'Email' },
  ];
  //by default, search by order id
  searchBy: string = 'orderId';
  //by default, search for orders with id = '' (all orders)
  searchKeyword: string = '';
  //admin or user -> different scope for each user role
  role = '';
  //nr of results
  totalResults = 0;
  displayedColumns: string[] = ['id', 'dueDate', 'doneDate', 'clientDeliveryEmail', 'signature', 'author', 'title', 'bvNr', 'isbnissn', 'actions'];
  dataSource: MatTableDataSource<OrderResponseDto> = new MatTableDataSource<OrderResponseDto>();
  //dataSource = [];
  @ViewChild(MatPaginator) paginator: MatPaginator | undefined;
  @ViewChild(MatSort) sort: MatSort | undefined;
  filterChange$ = new Subject<string>();
  subscriptions: Subscription[] = [];

  constructor(
    private router: Router,
    private loadingService: LoadingService,
    private alertService: AlertService,
    private tokenService: TokenService,
    private orderService: OrderService,
    private adminOrderService: AdminOrderService,
  ) { }


  ngOnInit(): void {
    this.subscriptions.push(
      this.tokenService.userAuthority$.subscribe(role => {
        this.role = role;
      })
    );
  }

  ngAfterViewInit(): void {
    // If the user changes the sort order, reset back to the first page.
    this.sort.sortChange.subscribe(() => (this.paginator.pageIndex = 0));

    // Merge paginator, sort, and filter changes into a single observable
    const trigger$ = merge(
      this.sort.sortChange,
      this.paginator.page,
      this.filterChange$.pipe(debounceTime(300)) // Debounce search inputs to avoid frequent calls
    );

    this.subscriptions.push(
      trigger$.pipe(
        startWith({}),
        switchMap(() => {
          this.loadingService.isLoading.next(true);
          const params = this.updateRequestParams();
          return this.handleSearch(params).pipe(
            catchError(() => {
              this.loadingService.isLoading.next(false);
              this.alertService.showAlert('Failed to search orders. Please try again later.', AlertType.ERROR );
              return of({ totalElements: 0, content: [] });
            })
          );
        }),
        map(data => {
          this.loadingService.isLoading.next(false);
          this.totalResults = data.totalElements || 0;
          return data.content || [];
        })
      ).subscribe(data => this.dataSource.data = data)
    );
  }

  //update params with searchBy, and pagination and sorting from dataSource
  private updateRequestParams() : SearchParams{
    return {
      [this.searchBy]: this.searchKeyword || '',
      'pageNo': this.paginator?.pageIndex || 0,
      'pageSize': this.paginator?.pageSize || 5,
      'sortBy': this.sort?.active === 'signature' ? 'document.signature' : this.sort?.active || 'dueDate',
      'sortDir': this.sort?.direction || 'asc'
    };
  }

  private handleSearch(params: any): Observable<any> {
    const serviceMap = {
      orderId: this.role === 'ROLE_ADMIN' ? this.adminOrderService.getAnyOrderByIdContaining(params) : this.orderService.getChairsOrderByIdContaining(params),
      orderSignature: this.role === 'ROLE_ADMIN' ? this.adminOrderService.getAnyOrderBySignatureContaining(params) : this.orderService.getChairsOrderBySignatureContaining(params),
      clientEmail: this.role === 'ROLE_ADMIN' ? this.adminOrderService.getAnyOrdersByClientEmailContaining(params) : this.orderService.getChairsOrderByClientEmailContaining(params),
    };
    return serviceMap[this.searchBy] || of({ totalElements: 0, content: [] });
  }

  viewOrder(order: OrderResponseDto) {
    this.router.navigate(['orders','archived' ,'view', order.id]);
  }

  onSearch(): void {
    if (this.paginator) {
      this.paginator.pageIndex = 0; // Reset paginator
    }
    //trigger new search
    this.filterChange$.next(this.searchKeyword);
  }

  ngOnDestroy(): void {
    this.subscriptions.forEach(sub => sub.unsubscribe());
  }
  
}


