import { AfterViewInit, ChangeDetectorRef, Component, OnDestroy, OnInit, ViewChild } from '@angular/core';
import { MatPaginator } from '@angular/material/paginator';
import { MatTableDataSource } from '@angular/material/table';
import { saveAs } from 'file-saver';
import * as Papa from 'papaparse';
import { catchError, map, merge, Observable, of, startWith, Subscription, switchMap, tap } from 'rxjs';
import { AlertService } from 'src/app/services/alert.service';
import { LoadingService } from 'src/app/services/loading.service';
import { FilterArchivedOrders$Params } from 'src/openapi/generated/fn/admin-order/filter-archived-orders';
import { OrderResponseDto } from 'src/openapi/generated/models';
import { AdminOrderService } from 'src/openapi/generated/services';


@Component({
  selector: 'app-statistics',
  templateUrl: './statistics.component.html',
  styleUrls: ['./statistics.component.css']
})
export class StatisticsComponent implements OnDestroy, AfterViewInit {

  faculties: string[] =  ['HB', 'TNZB', 'WSZB', 'EZB', 'JURA', 'NBG_FINDELGASSE', 'HB_DOD_SCAN_ZIMMER', 'SISIS_GRUPPE_TEST', 'TB_THEOLOGIE_KOCHSTRASSE', 'ERL_BISMARCKSTRASSE', "ALL"];
  statuses: string[] = ['SCANNED', 'REJECTED', 'ALL'];
  selectedFaculty: string = '';
  selectedStatus: string = '';
  selectedMonthStart: number = 0;  //defaults to 15 months ago
  selectedMonthEnd: number = 14; // defaults to the current month of current year
  displayedColumns: string[] = ['id', 'doneDate', 'clientDeliveryEmail', 'signature', 'author', 'title', 'bvNr', 'isbnissn'];

  private allData: OrderResponseDto[] = [];  // This will store all filtered data
  data: MatTableDataSource<OrderResponseDto> = new MatTableDataSource<OrderResponseDto>(); // for paginated data
  resultsLength = 0;
  hasOrders: boolean = true;
  @ViewChild(MatPaginator) paginator: MatPaginator;
  subscriptions: Subscription[] = [];


  constructor(
    private loadingService: LoadingService,
    private alertService: AlertService,
    private adminOrderService: AdminOrderService,
  ) {
  }

  
  ngAfterViewInit() { 
    //if table is not created bec has no orders, skip checking pagination
    if (this.paginator) {
      this.paginator.pageIndex = 0;

      this.subscriptions.push(
        this.paginator.page
          .pipe(
            tap(() => (this.loadingService.isLoading.next(true))),
            startWith(null),
            switchMap(() => {
              // Flip flag to show that loading has finished.
              this.loadingService.isLoading.next(false);
              if (this.allData.length > 0) {
                // Use cached data if available
                return of(this.paginateLocalData());
              }
              return this.filterArchivedOrders().pipe(catchError(() => of(null)));
            }),
          )
          .subscribe()
      );
    }
  }

  private filterArchivedOrders() : Observable<any> {
    let params: FilterArchivedOrders$Params = {
      startDate: this.getDateFromSelection(this.selectedMonthStart),
      endDate: this.getDateFromSelection(this.selectedMonthEnd),
      sortBy: 'doneDate',
      sortDir: 'desc',
      pageNo: 0,       // Set pageNo and pageSize to retrieve the full data
      pageSize: 99999 // High number to get all data
    };
    if (this.selectedFaculty) {
      params.faculty = this.selectedFaculty;
    } 
    if (this.selectedStatus) {
      params.status = this.selectedStatus;
    } 
    return this.adminOrderService.filterArchivedOrders(params)
    //;
    .pipe(
      map(data => {
        this.allData = data.content; // Cache the full data
        this.resultsLength = data.totalElements;
        this.hasOrders = data.totalElements > 0;
        this.paginateLocalData(); // Paginate based on the cached data
        return data;
      })
    );
  }


  // Local pagination logic based on cached data
  private paginateLocalData() {
    const pageIndex = this.paginator?.pageIndex || 0;
    const pageSize = this.paginator?.pageSize || 5;
    let paginatedData = [];
    if(this.allData != undefined || this.allData != null) {
      paginatedData = this.allData?.slice(pageIndex * pageSize, (pageIndex + 1) * pageSize);
      this.data = new MatTableDataSource(paginatedData);
    } else {
      this.data = new MatTableDataSource([]);
    }
    return paginatedData;
  }


  ngOnDestroy(): void {
    this.subscriptions.forEach(sub => sub.unsubscribe());
  }

  // Get label for the selected month
  getMonthLabel(value: number): string {
    if(value > 14 || value < 0) {
      return '';
    }
    const today = new Date();
    const selectedDate = new Date(today.getFullYear(), today.getMonth() - (14 - value), 1);
    return `${selectedDate.toLocaleString('default', { month: 'long' })} ${selectedDate.getFullYear()}`;
  }

  getDateFromSelection(value: number): string {
    if(value > 14 || value < 0) {
      throw new Error('Invalid month selected');
    }
    const today = new Date();
    return new Date(today.getFullYear(), today.getMonth() - (14 - value), 1).toDateString();
  }

  // Handle changes in the date interval / faculty / status
  filterResults() {
    if(this.data.paginator) {
      this.data.paginator.firstPage();
    }
    // Clear cache and fetch new data
    this.allData = [];
    if(this.paginator != undefined || this.paginator != null) {
      this.paginator.pageIndex = 0;
      this.paginator.pageSize = 5;
    }
    this.filterArchivedOrders().subscribe();
  }

  // Export filtered orders as CSV
  exportToCSV() {
    const csv = Papa.unparse(this.allData);
    const blob = new Blob([csv], { type: 'text/csv;charset=utf-8;' });
    saveAs(blob, 'orders.csv');
  }

}
