import { ManagementService } from './../../../../openapi/generated/services/management.service';
import { AfterViewInit, Component, OnDestroy, OnInit, ViewChild } from '@angular/core';
import { MatDialog } from '@angular/material/dialog';
import { MatPaginator } from '@angular/material/paginator';
import { MatSort } from '@angular/material/sort';
import { MatTableDataSource } from '@angular/material/table';
import { Subscription, merge, startWith, switchMap, catchError, of, map, Observable } from 'rxjs';
import { AlertType } from 'src/app/alert/alert-type.enum';
import { ConfirmationDialogComponent } from 'src/app/confirmation-dialog/confirmation-dialog.component';
import { AlertService } from 'src/app/services/alert.service';
import { LoadingService } from 'src/app/services/loading.service';
import { FaudokUserInfoResponseDto } from 'src/openapi/generated/models/faudok-user-info-response-dto';
import { GetChairsAvailableOrders$Params } from 'src/openapi/generated/fn/order/get-chairs-available-orders';

@Component({
  selector: 'app-user-management',
  templateUrl: './user-management.component.html',
  styleUrls: ['./user-management.component.css']
})
export class UserManagementComponent implements OnInit, OnDestroy, AfterViewInit {

  
  displayedColumns: string[] = ['id', 'name', 'email', 'role', 'availableFaculties', 'actions'];
  data: MatTableDataSource<FaudokUserInfoResponseDto> ;//= new MatTableDataSource();
  resultsLength = 0;
  @ViewChild(MatPaginator) paginator: MatPaginator;
  @ViewChild(MatSort) sort: MatSort;
  subscriptions: Subscription[] = [];


  constructor(
    private loadingService: LoadingService,
    private alertService: AlertService,
    private managementService: ManagementService,
    private dialog: MatDialog,
  ) {
    
    this.subscriptions.push(
      this.managementService.getUsers({pageNo: 0, pageSize: 5, sortBy: 'name', sortDir: 'asc'}).subscribe(data =>
       {
        this.data = new MatTableDataSource(data.content);
       }
      )
    );
  }

  ngOnInit(): void {
    // this.subscriptions.push(
    //   this.orderService.getChairsAvailableOrders({pageNo: 0, pageSize: 10, sortBy: 'dueDate', sortDir: 'asc'}).subscribe(data =>
    //    {
    //     data.totalElements !== 0 ? this.hasOrders = true : this.hasOrders = false;
    //     this.data = new MatTableDataSource(data.content);
    //    }
    //   )
    // );
  }

  ngOnDestroy(): void {
    this.subscriptions.forEach((sub) => sub.unsubscribe());
  }

  
  ngAfterViewInit() {
    //if table is not created bec has no orders, skip checking pagination and sorting
    if (this.sort && this.paginator) {
      // If the user changes the sort order, reset back to the first page.
      this.sort.sortChange.subscribe(() => (this.paginator.pageIndex = 0));
      this.subscriptions.push(
        //check if sort or pagination observable values changes to update the table
        merge(this.sort.sortChange, this.paginator.page)
          .pipe(
            startWith({}),
            switchMap(() => {
              this.loadingService.isLoading.next(true);
              return this.findAllUsers().pipe(catchError(() => of(null)));
            }),
            map(data => {
              // Flip flag to show that loading has finished.
              this.loadingService.isLoading.next(false);
              if (data === null) {
                return [];
              }
              // Only refresh the result length if there is new data. 
              this.resultsLength = data.totalElements;
              return data.content;
            }),
          )
          .subscribe(data => (this.data = new MatTableDataSource( data ) ))
      );
    }
  }

  private resetPaginationAndSorting(): void {
    this.paginator.pageIndex = 0;
    this.paginator.pageSize = 5;
    this.sort.active = 'name';
    this.sort.direction = 'asc';
  }

  applyFilter(event: Event) {
    const filterValue = (event.target as HTMLInputElement).value;
    this.data.filter = filterValue.trim().toLowerCase();
    if (this.data.paginator) {
      this.data.paginator.firstPage();
    }
  }

  private findAllUsers() : Observable<any> {
    let params: GetChairsAvailableOrders$Params = {
      pageNo: this.paginator.pageIndex,
      pageSize: this.paginator.pageSize,
      sortBy: this.sort.active,
      sortDir: this.sort.direction,
    };
    return this.managementService.getUsers(params);
  }


  deleteUser(user: FaudokUserInfoResponseDto) {
      const dialogRef = this.dialog.open(ConfirmationDialogComponent);   
      dialogRef.afterClosed().subscribe(result => {
        if (result) {
          this.loadingService.isLoading.next(true);
            this.managementService.deleteUserById({ id: user.id }).subscribe({
              next: (res) => {
                this.resetPaginationAndSorting();
                this.loadingService.isLoading.next(false);
                this.alertService.showAlert('User deleted!', AlertType.SUCCESS );
                this.subscriptions.push(
                  this.findAllUsers().subscribe(
                        res => {
                          this.loadingService.isLoading.next(false);
                          if (res !== null) {
                            this.data = new MatTableDataSource(res.content)
                            this.resultsLength = res.totalElements;
                          }
                        },
                        err => {
                          this.loadingService.isLoading.next(false);
                          this.alertService.showAlert('Something went wrong while fetching the data. Please try again later.', AlertType.ERROR);
                        }
                      )
                    )
              },
              error: (err) => {
                this.alertService.showAlert('Failed to delete the user. Please try again later.', AlertType.ERROR );
                this.loadingService.isLoading.next(false);
              }
            });
          } 
       }
     );  
  }

}
