import { Component, OnDestroy, OnInit } from '@angular/core';
import { FormGroup, FormBuilder, Validators } from '@angular/forms';
import { Router } from '@angular/router';
import { Subscription } from 'rxjs';
import { AlertType } from 'src/app/alert/alert-type.enum';
import { AlertService } from 'src/app/services/alert.service';
import { LoadingService } from 'src/app/services/loading.service';
import { ManagementService } from 'src/openapi/generated/services';


// Custom validator for weak passwords
function passwordStrengthValidator(control: any) {
  const weakPasswords = ["12345678", "password", "qwerty"];
  if (control.value && weakPasswords.includes(control.value)) {
    return { weakPassword: true };  // Return error if weak password
  }
  return null; // Return null if valid
}


@Component({
  selector: 'app-user-registration',
  templateUrl: './user-registration.component.html',
  styleUrls: ['./user-registration.component.css']
})
export class UserRegistrationComponent implements OnInit, OnDestroy {

  public registerForm!: FormGroup;
  faculties: string[] = ['HB', 'TNZB', 'WSZB', 'EZB','JURA', 'NBG_FINDELGASSE', 'HB_DOD_SCAN_ZIMMER', 'SISIS_GRUPPE_TEST', 'TB_THEOLOGIE_KOCHSTRASSE', 'ERL_BISMARCKSTRASSE'];
  roles: string[] = ["ROLE_ADMIN", "ROLE_USER"];
  registerRequest = { email: '', name: '', password: '', faculty: '', role: '' };
    // Track whether the main faculty selection is enabled
  mainFacultyEnabled: boolean = false;
  private subscriptions: Subscription[] = [];


  constructor(
    private fb: FormBuilder, 
    private managementService: ManagementService,
    private router: Router,
    private loadingService: LoadingService,
    private alertService: AlertService
  ) {}

  ngOnInit() {
    this.registerForm = this.fb.group({
      name: ['', Validators.required],
      password: ['', [Validators.required, Validators.minLength(8), Validators.maxLength(62), passwordStrengthValidator]],
      confirmPassword: ['', Validators.required],
      email: ['', [Validators.required, Validators.email]],
      availableFaculties: [[],  Validators.required],
      mainFaculty: ['', Validators.required],
      role: ['', Validators.required],
    }, { validator: this.passwordMatchValidator });

    // Watch availableFaculties changes to update the mainFaculty logic
    this.registerForm.get('availableFaculties')?.valueChanges.subscribe((faculties: string[]) => {
      this.updateMainFacultySelection(faculties);
    });
  }

  updateMainFacultySelection(faculties: string[]) {
    if (faculties.length === 1) {
      // Automatically select the main faculty if only one is available
      this.registerForm.get('mainFaculty')?.setValue(faculties[0]);
      this.mainFacultyEnabled = false; // Disable manual selection
    } else if (faculties.length > 1) {
      // Enable manual selection if more than one faculty is selected
      this.mainFacultyEnabled = true;
      // Reset mainFaculty if it's not in the selected faculties
      const mainFaculty = this.registerForm.get('mainFaculty')?.value;
      if (!faculties.includes(mainFaculty)) {
        this.registerForm.get('mainFaculty')?.setValue('');
      }
    } else {
      // Reset the main faculty if no faculty is selected
      this.registerForm.get('mainFaculty')?.setValue('');
      this.mainFacultyEnabled = false;
    }
  }

  passwordMatchValidator(control: FormGroup): { [key: string]: boolean } | null {
    const password = control.get('password').value;
    const confirmPassword = control.get('confirmPassword').value;
    
    if (password !== confirmPassword) {
      control.get('confirmPassword').setErrors({ passwordsDontMatch: true }); // Sets error on the confirmPassword control
      return { passwordsDontMatch: true }; // Sets error at the group level
    } else {
      control.get('confirmPassword').setErrors(null); // Clears the error on confirmPassword
      return null; // Clears the group-level error
    }
  }
  

  onSubmit() {
    if (this.registerForm.valid) {
      this.loadingService.isLoading.next(true);
      const {confirmPassword, ...rest} = this.registerForm.value;
      Object.assign(this.registerRequest, rest);

      this.subscriptions.push(
        this.managementService.register({ body: this.registerRequest }).subscribe({
          next: (res) => {
            this.loadingService.isLoading.next(false);
            this.alertService.showAlert('Registration request sent. Please check your email for account details.', AlertType.SUCCESS);
            this.router.navigate(['orders']);
          },
          error: (err) => {
            this.loadingService.isLoading.next(false);
            this.alertService.showAlert('Something went wrong. Please try again.', AlertType.ERROR);
          }
        })
      );
      this.registerForm.reset();
    }
  }

  ngOnDestroy() {
    this.subscriptions.forEach(sub => sub.unsubscribe());
  }

}
